@@ -29,17 +29,20 @@ const Lang = imports.lang;
 const Application = imports.application;
 const ContactPlace = imports.contactPlace;
 const Geoclue = imports.geoclue;
-const MapSource = imports.mapSource;
+const Location = imports.location;
 const MapWalker = imports.mapWalker;
 const Place = imports.place;
 const PlaceMarker = imports.placeMarker;
+const StoredRoute = imports.storedRoute;
 const TurnPointMarker = imports.turnPointMarker;
 const UserLocationMarker = imports.userLocationMarker;
 const Utils = imports.utils;
 
 const MapType = {
-    STREET:   'MapsStreetSource',
-    AERIAL:  'MapsAerialSource'
+    STREET:  Champlain.MAP_SOURCE_OSM_MAPQUEST,
+    AERIAL:  Champlain.MAP_SOURCE_OSM_AERIAL_MAP,
+    CYCLING: Champlain.MAP_SOURCE_OSM_CYCLE_MAP,
+    TRANSIT: Champlain.MAP_SOURCE_OSM_TRANSPORT_MAP
 };
 
 const MapMinZoom = 2;
@@ -74,6 +77,7 @@ const MapView = new Lang.Class({
         this.view = this._initView();
         this._initLayers();
 
+        this._factory = Champlain.MapSourceFactory.dup_default();
         this.setMapType(MapType.STREET);
 
         this._updateUserLocation();
@@ -146,15 +150,8 @@ const MapView = new Lang.Class({
         if (this.view.map_source.id === mapType)
             return;
 
-        if (mapType === MapType.AERIAL)
-            this.view.map_source = MapSource.createAerialSource();
-        else
-            this.view.map_source = MapSource.createStreetSource();
-
-        if (!this._attribution) {
-                this._attribution = new MapSource.AttributionLogo(this.view);
-                this.view.add_child(this._attribution);
-        }
+        let source = this._factory.create_cached_source(mapType);
+        this.view.map_source = source;
     },
 
     gotoUserLocation: function(animate) {
@@ -201,8 +198,8 @@ const MapView = new Lang.Class({
     _gotoBBox: function(bbox) {
         let [lat, lon] = bbox.get_center();
         let place = new Place.Place({
-            location: new Geocode.Location({ latitude  : lat,
-                                             longitude : lon }),
+            location: new Location.Location({ latitude  : lat,
+                                              longitude : lon }),
             bounding_box: new Geocode.BoundingBox({ top    : bbox.top,
                                                     bottom : bbox.bottom,
                                                     left   : bbox.left,
@@ -241,16 +238,44 @@ const MapView = new Lang.Class({
         this._gotoBBox(contact.bounding_box);
     },
 
-    showSearchResult: function(place) {
+    _showStoredRoute: function(stored) {
+        let query = Application.routeService.query;
+        let route = Application.routeService.route;
+
+        Application.routeService.storedRoute = stored.route;
+
+        let resetId = route.connect('reset', function() {
+            route.disconnect(resetId);
+            query.freeze_notify();
+
+            let storedLast = stored.places.length - 1;
+            query.points[0].place = stored.places[0];
+            query.points[1].place = stored.places[storedLast];
+
+            for (let i = 1; i < storedLast; i++) {
+                let point = query.addPoint(i);
+                point.place = stored.places[i];
+            }
+
+            query.thaw_notify();
+        });
+        route.reset();
+    },
+
+    showSearchResult: function(place, animation) {
         this._placeLayer.remove_all();
+
+        if (place instanceof StoredRoute.StoredRoute) {
+            this._showStoredRoute(place);
+            return;
+        }
+
         this.routeVisible = false;
         let placeMarker = new PlaceMarker.PlaceMarker({ place: place,
                                                         mapView: this });
 
         this._placeLayer.add_marker(placeMarker);
-        placeMarker.goToAndSelect(true);
-
-        return placeMarker;
+        placeMarker.goToAndSelect(animation);
     },
 
     showRoute: function(route) {
