@@ -26,6 +26,7 @@ const Clutter = imports.gi.Clutter;
 const Geocode = imports.gi.GeocodeGlib;
 const Lang = imports.lang;
 
+const Location = imports.location;
 const Utils = imports.utils;
 
 const _MAX_DISTANCE = 19850; // half of Earth's circumference (km)
@@ -57,15 +58,8 @@ const MapWalker = new Lang.Class({
         let zoom;
 
         if (this._boundingBox !== null) {
-            let max = this._view.max_zoom_level;
-            let min = this._view.min_zoom_level;
-            for (let i = max; i >= min; i--) {
-                let zoomBox = this._view.get_bounding_box_for_zoom_level(i);
-                if (this._boxCovers(zoomBox)) {
-                    zoom = i;
-                    break;
-                }
-            }
+            this._view.zoom_level = this._view.max_zoom_level;
+            this._view.ensure_visible(this._boundingBox, false);
         } else {
             switch (this.place.place_type) {
             case Geocode.PlaceType.STREET:
@@ -88,10 +82,10 @@ const MapWalker = new Lang.Class({
                 zoom = 11;
                 break;
             }
+            this._view.zoom_level = zoom;
+            this._view.center_on(this.place.location.latitude,
+                                 this.place.location.longitude);
         }
-        this._view.zoom_level = zoom;
-        this._view.center_on(this.place.location.latitude,
-                             this.place.location.longitude);
     },
 
     goTo: function(animate) {
@@ -99,11 +93,9 @@ const MapWalker = new Lang.Class({
         this._mapView.emit('going-to');
 
         if (!animate) {
-            this._view.animate_zoom = false;
-            this.zoomToFit();
-            this._view.animate_zoom = true;
+            this._view.center_on(this.place.location.latitude,
+                                 this.place.location.longitude);
             this.emit('gone-to');
-
             return;
         }
 
@@ -117,8 +109,8 @@ const MapWalker = new Lang.Class({
 
         this._view.goto_animation_mode = Clutter.AnimationMode.EASE_IN_CUBIC;
 
-        let fromLocation = new Geocode.Location({ latitude: this._view.get_center_latitude(),
-                                                  longitude: this._view.get_center_longitude() });
+        let fromLocation = new Location.Location({ latitude: this._view.get_center_latitude(),
+                                                   longitude: this._view.get_center_longitude() });
         this._updateGoToDuration(fromLocation);
 
         Utils.once(this._view, 'animation-completed', (function() {
