@@ -28,16 +28,12 @@ const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 
 const Application = imports.application;
+const Location = imports.location;
 const Place = imports.place;
 const PlaceStore = imports.placeStore;
 const SearchPopup = imports.searchPopup;
 const Utils = imports.utils;
 
-Geocode.Location.prototype.equals = function(location) {
-    return (location.latitude === this.latitude &&
-            location.longitude === this.longitude);
-};
-
 // Matches coordinates string with the format "<lat>, <long>"
 const COORDINATES_REGEX = /^\s*(\-?\d+(?:\.\d+)?)\s*,\s*(\-?\d+(?:\.\d+)?)\s*$/;
 
@@ -57,7 +53,7 @@ const PlaceEntry = new Lang.Class({
         if (!this._place && !p)
             return;
 
-        if (this._place && p && this._place.location.equals(p.location))
+        if (this._place && p && this._locEquals(this._place, p))
             return;
 
         if (p) {
@@ -96,6 +92,9 @@ const PlaceEntry = new Lang.Class({
         let parseOnFocusOut = props.parseOnFocusOut;
         delete props.parseOnFocusOut;
 
+        this._matchRoute = props.matchRoute || false;
+        delete props.matchRoute;
+
         this.parent(props);
 
         this._filter = new Gtk.TreeModelFilter({ child_model: Application.placeStore });
@@ -134,6 +133,11 @@ const PlaceEntry = new Lang.Class({
         }
     },
 
+    _locEquals: function(placeA, placeB) {
+        return (placeA.location.latitude === placeB.location.latitude &&
+                placeA.location.longitude === placeB.location.longitude);
+    },
+
     _createPopover: function(numVisible, maxChars) {
         let popover = new SearchPopup.SearchPopup({ num_visible:   numVisible,
                                                     relative_to:   this,
@@ -161,6 +165,10 @@ const PlaceEntry = new Lang.Class({
 
     _completionVisibleFunc: function(model, iter) {
         let place = model.get_value(iter, PlaceStore.Columns.PLACE);
+        let type = model.get_value(iter, PlaceStore.Columns.TYPE);
+
+        if (!this._matchRoute && type === PlaceStore.PlaceType.FAVORITE_ROUTE)
+            return false;
 
         if (place !== null)
             return place.match(this.text);
@@ -180,8 +188,8 @@ const PlaceEntry = new Lang.Class({
             let longitude = parseFloat(match[2]);
 
             if (this._validateCoordinates(latitude, longitude)) {
-                return new Geocode.Location({ latitude: latitude,
-                                              longitude: longitude });
+                return new Location.Location({ latitude: latitude,
+                                               longitude: longitude });
             } else
                 return null;
         } else
@@ -194,6 +202,20 @@ const PlaceEntry = new Lang.Class({
             return true;
         }
 
+        if (this.text.startsWith('geo:')) {
+            let location = new Geocode.Location();
+
+            try {
+                location.set_from_uri(this.text);
+                this.place = new Place.Place({ location: location });
+            } catch(e) {
+                let msg = _("Failed to parse Geo URI");
+                Application.notificationManager.showMessage(msg);
+            }
+
+            return true;
+        }
+
         let parsedLocation = this._parseCoordinates(this.text);
         if (parsedLocation) {
             this.place = new Place.Place({ location: parsedLocation });
